import sys
import inspect

from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from admin_tools.dashboard import modules

from pbx.ivr.models import NodoIvr, TimeSlot
from pbx.ivr.forms import TimeTableForm


class PbxDashboard(modules.Group):
    def __init__(self, **kwargs):
        children = [
                PbxModels(),
                RoutingModels(),
                TimeTable(),
                ChiusureTable(),
                IvrTable(),
                UtentiModels(),
                CdrModels(),
                ]
        # aggiungo ai children eventuali classi specificate nei local settings
        for l in  getattr(settings, 'PBX_DEVICES_DASHBOARDS'):
            try:
                mod = __import__(l[0], globals(), locals(), [l[1]])
                classe_childtab = getattr(mod, l[1])
                children.append(classe_childtab())
            except:
                raise ImportError
        kwargs =  {
            'title': "Pbx",
            'display': "tabs",
            'children': children
        }
        super(PbxDashboard, self).__init__(**kwargs)

class PbxModels(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': _("Internal Options"),
            'models': ('pbx.switch.models.Account',
                       'pbx.switch.models.AccountFunctions',
                       'pbx.switch.models.CallGroup',
                       'pbx.switch.models.VoiceMail',
                       'pbx.switch.models.Queue',
                       'pbx.switch.models.QueueMember',
                       'pbx.switch.models.MeetMe',
                       'pbx.switch.models.MeetMeUser',),
        }
        super(PbxModels, self).__init__(**kwargs)

class UtentiModels(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': _("Directory"),
            'models': ('pbx.utenti.models.BlacklistEntry',
                       'pbx.utenti.models.PersonalPhonebookEntry',
                       'pbx.utenti.models.SpeedDialEntry'),
        }
        super(UtentiModels, self).__init__(**kwargs)

class RoutingModels(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': _("Routing"),
            'models': ('pbx.routing.models.Rotta',
                       #'sctel.prefissi.models.Prefisso',
                       'sctel.prefissi.models.NomeDistretto',
                       #'sctel.linee.models.Terminazione',
                       'sctel.linee.models.Peer'),
        }
        super(RoutingModels, self).__init__(**kwargs)
        
class IvrTreelist(modules.ModelList):
    def __init__(self, **kwargs):
        self.template = 'admin_tools/pbx/ivr/treelist.html'
        kwargs =  {
            'title': _("OldGuiIvr"),
            'template' : self.template,
#            'children' : self.children,
#            'models': ('pbx.ivr.models.NodoIvr',
#                       'pbx.ivr.models.TimeSlot',),
        }
        super(IvrTreelist, self).__init__(**kwargs)
        
    def init_with_context(self, context):
        super(IvrTreelist, self).init_with_context(context)
        #request = context['request']
        context['qs_nodi'] = NodoIvr.objects.all()
        context['qs_timeslots'] = TimeSlot.objects.all()
        
class TimeTable(modules.ModelList):
    def __init__(self, **kwargs):
        self.template = 'admin_tools/pbx/ivr/timetable.html'
        kwargs = {
            'title': _("Orari Apertura"),
            'template':self.template,}
        super(TimeTable, self).__init__(**kwargs)
        
    def init_with_context(self, context):
        super(TimeTable, self).init_with_context(context)
        form = TimeTableForm()
        context['form'] = form
        # servono per il rendering corretto della tabella con un loop nei field
        context['abilitato_counter'] = (0, 5, 10, 15, 20, 25, 30)
        context['dallemattino_counter'] = (1, 6, 11, 16, 21, 26, 31)
        context['allemattino_counter'] = (2, 7, 12, 17, 22, 27, 32)
        context['dallepomeriggio_counter'] = (3, 8, 13, 18, 23, 28, 33)
        context['allepomeriggio_counter'] = (4, 9, 14, 19, 24, 29, 34)
        
class ChiusureTable(modules.ModelList):
    def __init__(self, **kwargs):
        self.template = 'admin_tools/pbx/ivr/chiusuretable.html'
        kwargs = {
            'title': _("Chiusure extra"),
            'template': self.template,}
        super(ChiusureTable, self).__init__(**kwargs)
        
    def init_with_context(self, context):
        super(ChiusureTable, self).init_with_context(context)
        context['timeslot_chiusure_extra'] = TimeSlot.objects.exclude(
                                                        dal_day__exact=None)
        
class IvrTable(modules.ModelList):
    def __init__(self, **kwargs):
        self.template = 'admin_tools/pbx/ivr/ivrtable.html'
        kwargs =  {
            'title': _("Ivr"),
            'template' : self.template,
        }
        super(IvrTable, self).__init__(**kwargs)
        
    def init_with_context(self, context):
        super(IvrTable, self).init_with_context(context)
        context['nodo_default'] = NodoIvr.objects.get(nome='default')
        context['nodo_lavorativi'] = NodoIvr.objects.get(nome='lavorativi')
        context['nodo_pausa'] = NodoIvr.objects.get(nome='pausa')
        context['nodi_apertura'] = NodoIvr.objects.filter(padre=context['nodo_lavorativi'])
        context['nodi_chiusura'] = NodoIvr.objects.filter(padre=context['nodo_default'])
        context['qs_nodi'] = NodoIvr.objects.all()
        
    
class CdrModels(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs = {
            'title': _("Reports"),
            'models': ('pbx.reports.models.Cdr',),
        }
        super(CdrModels, self).__init__(**kwargs)
