# -*- coding: utf-8 -*-
'''
Created on 14/feb/2012

@author: simone cittadini
'''
from django.db import models


TONI = (('0', '0'), ('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5'),
        ('6', '6'), ('7', '7'), ('8', '8'), ('9', '9'), ('*', '*'), ('#', '#'))

CHOICES_WEEDKAYS = ((0, 'sempre'), (1, 'lunedi'), (2, 'martedi'),
                    (3, 'mercoledi'), (4, 'giovedi'), (5, 'venerdi'),
                    (6, 'sabato'), (7, 'domenica'), (8, 'weekend'))

DICT_WEEKDAYS = {0 : 'sempre',
                 1 : 'lunedi',
                 2 : 'martedi',
                 3 : 'mercoledi',
                 4 : 'giovedi',
                 5 : 'venerdi',
                 6 : 'sabato',
                 7 : 'domenica',
                 8 : 'weekend',}


class FileNodoInRegistrazione(models.Model):
    filename = models.CharField(max_length=32)
    
    @staticmethod
    def get_filename():
        return FileNodoInRegistrazione.objects.all()[0].filename
    
    @staticmethod
    def set_filename(filename):
        try:
            f = FileNodoInRegistrazione.objects.all()[0]
            f.filename = filename
            f.save()
        except Exception, e:
            FileNodoInRegistrazione.objects.create(filename=filename)
    
class NodoIvr(models.Model):
    nome = models.CharField(max_length=32)
    padre = models.ForeignKey('self', blank=True, null=True)
    tono = models.CharField(max_length=1, choices=TONI, blank=True, null=True)
    filename = models.CharField(max_length=32, blank=True, null=True)
    timeout = models.PositiveSmallIntegerField(default=0)
    extension = models.CharField(max_length=32, blank=True, null=True)

    class Meta:
        ordering = (('tono',))
        unique_together = (('padre', 'tono'),)


    def __unicode__(self):
        return self.nome
    
    @property
    def unicode(self):
        return self.__unicode__()
    
    @property
    def curl_string(self):
        if self.padre:
            padre = self.padre.pk
        else:
            padre = '0'
        if self.filename:
            return "%s__%s__%s__%s" % (self.pk, self.filename, self.timeout, self.extension)
        else:
            return "%s__NF__%s__%s" % (self.pk, self.timeout, self.extension)
    
    def save(self, *args, **kwargs):
        super(NodoIvr, self).save(*args, **kwargs)
        if self.filename:
            FileNodoInRegistrazione.set_filename(self.filename)


class TimeSlot(models.Model):
    dal_day = models.DateField(blank=True, null=True)
    al_day = models.DateField(blank=True, null=True)
    weekday = models.PositiveIntegerField(default=0, choices=CHOICES_WEEDKAYS)
    dalle_mattina = models.TimeField()
    alle_mattina = models.TimeField()
    dalle_pomeriggio = models.TimeField()
    alle_pomeriggio = models.TimeField()
    radice = models.ForeignKey(NodoIvr)
    attivo = models.BooleanField(default=True)

    def __unicode__(self):
        return "%s , from %s to %s" % (DICT_WEEKDAYS[self.weekday], self.inizio, self.fine)
    
    @property
    def periodo(self):
        return DICT_WEEKDAYS[self.weekday]
    
    @property
    def figli(self):
        return NodoIvr.objects.filter(padre__exact=self.radice)
    
    def save(self, *args, **kwargs):
        if not self.dal_day or not self.al_day:
            self.dal_day = None
            self.al_day = None
        super(TimeSlot, self).save(*args, **kwargs)
