# -*- coding: utf-8 -*-
'''
Created on 1/ago/2011

@author: simone cittadini
'''
from string import capwords

from django.db import models

#from sctel.prefissi.models import NomeDistretto, Prefisso
#from sctel.linee.models import Terminazione, Peer
from django.db.models.signals import m2m_changed

from pbx.core.exceptions import IncoerenzaNelleMaschere


MAX_PREFIX_LENGTH = 16
MIN_PREFIX_LENGTH = 1

DEST_TYPES_CHOICES = ((1,'FIX'), (2,'MOB'), (3,'NGN'))
DEST_TYPES = {'FIX' : 1, 'MOB' : 2, 'NGN': 3}

PROTOCOLLI = (
    ("SIP", "SIP"),
    ("IAX", "IAX"),
#    ("ZAP", "ZAP"), # non servono finchè restiamo su centralini aziendali
#    ("323", "323"),
)

#class Maschera(models.Model): # non serve finchè restiamo su centralini aziendali
#    """
#    Raccolta di associazioni fra prefissi e distretti.
#    """
#    nome = models.CharField(max_length=32, unique=True)
#    base = models.BooleanField(default=False)
#
#    class Meta:
#        ordering = ('base', 'nome',)
#        verbose_name_plural = 'Maschere'
#
#    def __unicode__(self):
#        return self.nome
#
#    def save(self, *args, **kwargs):
#        # Non può esistere più di una maschera base
#        if self.base:
#            if Maschera.objects.filter(
#                                base__exact=True).exclude(
#                                nome__exact=self.nome).count() == 1:
#                raise IncoerenzaNelleMaschere(
#                                    "Può esistere solo una maschera base")
#        super(Maschera, self).save(*args, **kwargs)
#
#    @staticmethod
#    def get_maschera_base():
#        try:
#            return Maschera.objects.get(base=True)
#        except Maschera.DoesNotExist:
#            return None
        
class Prefisso(models.Model):
    """
    Rappresentazione di un prefisso internazionale.
    """
    pre = models.CharField(max_length=16, unique=True)

    class Meta:
        ordering = ('pre',)
        verbose_name_plural = 'Prefissi'

    def __unicode__(self):
        return self.pre

    def save(self, *args, **kwargs):
        if self.pre == None or len(self.pre) < MIN_PREFIX_LENGTH:
            raise IncoerenzaNelleMaschere("Prefisso non specificato.")
        super(Prefisso, self).save(*args, **kwargs)

    @staticmethod
    def get_prefisso(dst, min_length=MIN_PREFIX_LENGTH,
                     max_length=MAX_PREFIX_LENGTH):
        if len(dst) < max_length:
            max_length = len(dst)
        for i in range(min_length, max_length):
            try:
                p = Prefisso.objects.get(pre=dst[0:max_length - i])
                return p
            except:
                pass
        raise  # TODO 

    def get_distretto_base(self):
#        maschera_base = Maschera.get_maschera_base()
#        try:
#            return self.nomi_distretto.get(maschera=maschera_base)
#        except:
#            return None
        # qui ho lasciato una relazione uno a molti anche se in un centralino
        # posso dare per scontato non verrà mai usata
        return self.nomi_distretto[0]

import django
if django.VERSION[1] > 2:
    from django.dispatch import receiver
    @receiver(m2m_changed)
    def callback_m2m_changed(sender, **kwargs):
        if str(sender) == "<class 'sctel.prefissi.models.NomeDistretto_prefissi'>":
            if kwargs['action'] == 'pre_add':
                # La maschera associata non può contenere prefissi duplicati;
                #  prefissi già esistenti sotto la stessa maschera vengono rimossi
                #  dall'attuale nome distretto.
                #maschera = kwargs['instance'].maschera 
                ### non abbiamo maschere prefissi in un centralino
                for pk in kwargs['pk_set']:
                    prefisso = Prefisso.objects.get(pk=pk)
                    for nome_distretto in prefisso.nomi_distretto.all():
                        #if nome_distretto.maschera == maschera:
                        nome_distretto.prefissi.remove(prefisso)
                            
class NomeDistretto(models.Model):
    """
    Associazione fra un distretto / carrier e un insieme di prefissi.
    """
    nome = models.CharField(max_length=64)
#    tipo = models.PositiveIntegerField(choices=DEST_TYPES_CHOICES)
    prefissi = models.ManyToManyField(Prefisso, blank=True, null=True,
                                      related_name='nomi_distretto')
#    maschera = models.ForeignKey(Maschera)

    class Meta:
        ordering = ('nome',)
        #unique_together = ('nome', 'maschera',)
        verbose_name = 'Direttrici'
        verbose_name_plural = 'Direttrici'

    def __unicode__(self):
        return "%s - %s" % (self.maschera, self.nome)

    def save(self, *args, **kwargs):
        self.nome = capwords(self.nome.strip())
        super(NomeDistretto, self).save(*args, **kwargs)
        
class Linea(models.Model):
    """
    Classe base per i tipi di linea.
    """
    nome = models.CharField(max_length=32, unique=True)

    def __unicode__(self):
        return "%s" % self.nome

    @staticmethod
    def get_ingresso(dst, len_tech_prefix=None):
        """
        Dato un numero chiamato ritorna la linea corrispondente.
        """
        pass # in un centralino possiamo dare per scontato l'essere mono linea
#        if len_tech_prefix:
#            try:
#                t = TechPrefix.objects.get(codice=dst[0:len_tech_prefix])
#                return t.ingresso
#            except TechPrefix.DoesNotExist:
#                raise  ## TODO
#        else:
#            for i in range(len(dst)):
#                try:
#                    t = TechPrefix.objects.get(codice=dst[0:i])
#                    return t.ingresso
#                except:
#                    pass
#        raise  ## TODO

    def get_terminazione(self, distretto):
        """
        Ritorna l'ultima linea della rotta.
        """
        raise NotImplementedError
    
    def get_backup(self, distretto):
        """
        Per una linea di rivendita ritorna una terminazione alternativa,
        altrimenti ritorna None.
        """
        raise NotImplementedError
    
    def get_dial_string(self, dst_number, pre=None):
        """
        Per una linea di Terminazione ritorna gli argomenti per un comando dial
        dello switch, relativi a uno dei peer disponibili.
        """
        raise NotImplementedError
    
class Terminazione(Linea):
    """
    Rappresenta una terminazione fisica su un peer.
    """
    class Meta:
        verbose_name_plural = 'Terminazioni'
        
    def get_terminazione(self, distretto):
        return self

    def get_dial_string(self, dst_number, pre=None):
        try:
            p = Peer.objects.filter(terminazione__exact=self).order_by('?')[:1]
            if p.protocollo == 'SIP':
                pass
            else:
                raise NotImplementedError
        except:
            raise
        
class Peer(models.Model):
    nome = models.CharField(max_length=80)
    protocollo = models.CharField(max_length=3, choices=PROTOCOLLI,
                                  default='SIP')
    address = models.CharField(max_length=64)
    port = models.CharField(max_length=6, default='5060')
    username = models.CharField(max_length=64, blank=True, null=True)
    secret = models.CharField(max_length=64, blank=True, null=True)
#    strip_leading_zeroes = models.BooleanField(default=True)
#    strip_local_countrycode = models.BooleanField(default=False)
#    tech_prefix = models.CharField(max_length=12, blank=True)
#    max_calls = models.PositiveIntegerField(default=0)
    terminazione = models.ForeignKey(Terminazione, editable=False, blank=True, null=True)

    class Meta:
        verbose_name = 'Flusso'
        verbose_name_plural = 'Flussi'
        ordering = ['terminazione', 'protocollo', 'address']

    def __unicode__(self):
        return "%s-%s-%s" % (self.terminazione, self.protocollo, self.address)

    def save(self, *args, **kwargs):
        from pbx.switch.models import Account
        terminazione,_ = Terminazione.objects.get_or_create(nome=self.nome)
        self.terminazione = terminazione
        account, created = Account.objects.get_or_create(name=self.username,
                                                         type='peer',
                                    defaults={'context' : 'from-pstn',
                                              'secret' : self.secret,
                                              'host' : self.address,
                                              'nat' : 'yes',
                                              'defaultuser' : self.nome})
        if not created:
            account.secret = self.secret
            account.host = self.address
            account.defaultuser = self.nome
            account.save()

        super(Peer, self).save(*args, **kwargs)


class Rotta(models.Model):
    direttrice = models.ForeignKey(NomeDistretto, unique=True)
    terminazione = models.ForeignKey(Terminazione)

    class Meta:
        verbose_name_plural = 'Rotte'
        
    def __unicode__(self):
        return "%s - %s" % (self.destinazione.nome, self.terminazione)

    
    @staticmethod
    def get_dial_string(dst):
        prefisso = Prefisso.get_prefisso(dst)
        distretto = prefisso.get_distretto_base()
        try:
            rotta = Rotta.objects.get(destinazione=distretto)
            return rotta.terminazione.get_dial_string(dst)
        except Exception, e:
            return None
