# -*- coding: utf-8 -*-
'''
Created on 2/ago/2011

@author: simone cittadini
'''
from django.contrib import admin
from django import forms
from django.utils.translation import ugettext_lazy as _
from django.conf.urls.defaults import patterns, url
from django.core.urlresolvers import reverse

from models import Account, AccountFunctions, VoiceMail, MeetMe, CallGroup, \
    Queue, QueueMember, RecallBooking, MeetMeUser#, DevicePolycom#, DeviceCisco, DeviceCiscoDual#, SedeServer

from jumbo.admin.options import ExtendibleModelAdmin
from pbx.switch.views import device_restart

class QueueSearchForm(forms.ModelForm):
    class Meta:
        model = Queue
        display_fields = (
            'name__icontains',
        )

    def __init__(self, *args, **kws):
        super(QueueSearchForm, self).__init__(*args, **kws)
        self.fields['name__icontains'] = forms.CharField(label=_("Number"), required=False)


class QueueAdmin(ExtendibleModelAdmin):
    list_display = ('name',
                    'get_detail_icon',
                    'get_edit_icon',
                    'get_delete_icon',
                    )
    
    search_form = QueueSearchForm
    
    fieldsets =     (
        (None, {'fields': ('name', 'musiconhold', 'timeout', 'retry', 'strategy'),}),
                 )
    
class QueueMemberAdminForm(forms.ModelForm):
    membername = forms.ChoiceField()
    queue_name = forms.ChoiceField()
    
    class Meta:
        model = QueueMember
        
    def __init__(self, *args, **kwargs):
        super(QueueMemberAdminForm, self).__init__(*args, **kwargs)
        self.fields['membername'].choices = ((a.name, a.utentepbx) for a in Account.objects.all())
        self.fields['queue_name'].choices = ((q.name, q.name) for q in Queue.objects.all())
    
class QueueMemberAdmin(ExtendibleModelAdmin):
    list_display = ('utentepbx',
                    'get_detail_icon',
                    'get_edit_icon',
                    'get_delete_icon',
                    )
    form = QueueMemberAdminForm

class AccountSearchForm(forms.ModelForm):
    class Meta:
        model = Account
        display_fields = (
            'utentepbx__nome__icontains','name__icontains',
        )

    def __init__(self, *args, **kws):
        super(AccountSearchForm, self).__init__(*args, **kws)
        self.fields['name__icontains'] = forms.CharField(label=_("Number"), required=False)
        self.fields['utentepbx__nome__icontains'] = forms.CharField(label=_("Utentepbx"), required=False)

class AccountAdmin(ExtendibleModelAdmin):
    list_display = ('utentepbx',
                    'name',
                    'get_detail_icon',
                    'get_edit_icon',
                    'get_delete_icon',
                    )
    readonly_fields = ('ipaddr', 'port', 'useragent')
    
    fieldsets = (
        (_('stato attuale'), {'fields': ('ipaddr', 'port', 'useragent'),
                                         'classes': ('collapse',)}),
        (None, {'fields': ('utentepbx', 'name', 'callerid', 'secret', 'nat', 
                           'fax'),}),
                 )
    search_form = AccountSearchForm
    
    # non voglio poter aggiungere un utentepbx da qui, lascia aperta la strada
    # a casini.
    class Media:
        css = { 'all' : ('css/no-addanother-button.css',) }
    
class AccountFunctionsSearchForm(forms.ModelForm):
    class Meta:
        model = AccountFunctions
        display_fields = (
            'account__utentepbx__nome__icontains', 'account__name__icontains',
        )

    def __init__(self, *args, **kws):
        super(AccountFunctionsSearchForm, self).__init__(*args, **kws)
        self.fields['account__name__icontains'] = forms.CharField(label=_("Numero"), required=False)
        self.fields['account__utentepbx__nome__icontains'] = forms.CharField(label=_("Utente"), required=False)
    
class AccountFunctionsAdmin(ExtendibleModelAdmin):
    list_display = ('account',
                    'get_detail_icon',
                    'get_edit_icon',
                    'get_delete_icon',
                    )
    
    search_form = AccountFunctionsSearchForm
    
    # non voglio poter aggiungere un account da qui, lascia aperta la strada
    # a casini.
    class Media:
        css = { 'all' : ('css/no-addanother-button.css',) }
    
class VoiceMailSearchForm(forms.ModelForm):
    class Meta:
        model = VoiceMail
        display_fields = (
            'email__icontains', 'mailbox__icontains',
        )

    def __init__(self, *args, **kws):
        super(VoiceMailSearchForm, self).__init__(*args, **kws)
        self.fields['email__icontains'] = forms.CharField(label=_("Email"), required=False)
        self.fields['mailbox__icontains'] = forms.CharField(label=_("Mailbox"), required=False)
    
class VoiceMailAdmin(ExtendibleModelAdmin):
    list_display = ('email',
                    'mailbox',
                    'get_detail_icon',
                    'get_edit_icon',
                    'get_delete_icon',
                    )
    
    search_form = VoiceMailSearchForm
    
class MeetMeAdmin(ExtendibleModelAdmin):
    list_display = ('confno',
                    'get_detail_icon',
                    'get_edit_icon',
                    'get_delete_icon',
                    )
    
    fieldsets = ((None, {'fields' : ('confno', 'starttime', 'endtime', 'pin',
                                     'adminpin', 'recordingfilename',
                                     'maxusers')}),)
    
class CallGroupSearchForm(forms.ModelForm):
    class Meta:
        model = CallGroup
        display_fields = (
            'exten__icontains',
        )

    def __init__(self, *args, **kws):
        super(CallGroupSearchForm, self).__init__(*args, **kws)
        self.fields['exten__icontains'] = forms.CharField(label=_("Exten"), required=False)
    
class CallGroupAdmin(ExtendibleModelAdmin):
    list_display = ('exten',
                    'get_detail_icon',
                    'get_edit_icon',
                    'get_delete_icon',
                    )

    search_form = CallGroupSearchForm
    
class RecallBookingAdmin(ExtendibleModelAdmin):
    list_display = ('timestamp',
                    'caller',
                    'called',
                    'get_delete_icon',
                    )



admin.site.register(Account, AccountAdmin)
admin.site.register(VoiceMail, VoiceMailAdmin)
admin.site.register(MeetMe, MeetMeAdmin)
admin.site.register(MeetMeUser)
admin.site.register(AccountFunctions, AccountFunctionsAdmin)
admin.site.register(CallGroup, CallGroupAdmin)
admin.site.register(Queue, QueueAdmin)
admin.site.register(QueueMember, QueueMemberAdmin)
admin.site.register(RecallBooking, RecallBookingAdmin)
