# -*- coding: utf-8 -*-
'''
Created on 27/gen/2012

@author: simone cittadini
'''
import os, sys
import re
import httplib, urllib
import shlex
import subprocess
import time
import base64
import threading


from lxml import etree
from django.db import models
from django.db.models.query import QuerySet
from django.utils.translation import ugettext_lazy as _
from django.forms import fields
from django.contrib.contenttypes.models import ContentType

from django.template.loader import render_to_string
from jumbo.utils.functions import get_settings

from pbx.utenti.models import UtentePBX
from pbx.switch.models import Account, Device, SERVERURL, SERVERIP


CISCOIMAGESPATH = get_settings('switch.cisco.settings', 'CISCOIMAGESPATH')
CISCOWAITFORPUSH = get_settings('switch.cisco.settings', 'CISCOWAITFORPUSH')


class DeviceCisco(Device):
    
    def save(self, *args, **kwargs):
        super(DeviceCisco, self).save(*args, **kwargs)
        self.create_provision_file()
    
    def create_provision_file(self):
        account = self.account
        utente = account.utentepbx
        user = utente.get_user()
        try:
            phonelabel = "%s %s." % (user.last_name, user.first_name[0:1])
        except:
            phonelabel = 'no name set'
        if len(phonelabel) > 12:
            phonelabel = phonelabel[0:12]
        ctx_dict = {'server' : SERVERURL,
                    'processnodename' : SERVERIP,
                    'proxy' : SERVERIP,
                    'name' : self.account.name,
                    'password' : self.account.secret,
                    'phonelabel': phonelabel,
                    'deviceid' : self.pk}
        content = render_to_string('switch/cisco/provision.xml', ctx_dict)
        file = open("/srv/tftp/SEP%s.cnf.xml" % self.mac_address.replace(':', '').upper(),
                    "w")
        file.write(content)
        file.close()
    
    def pre_dial_action(self, cli, exten):
        print >> sys.stderr, "INTO CISCO PREDIAL %s %s" % (cli, exten)
        # se il cli sono io mando ai miei monitoranti la push 
        # allo stato di calling
        try:
            if DeviceCisco.objects.get(account__name=cli) == self:
                print >> sys.stderr, "%s si dichiara calling a %s" % (self.account.utentepbx, self.account.utentepbx.osservatori.all())
                for u in self.account.utentepbx.osservatori.all():
                    for a in u.accounts_utentepbx.all():
                        xml = { "XML" : '<CiscoIPPhoneExecute>\
                                <ExecuteItem URL="%s/switch/cisco_updatestatus_calling/%s/%s/%s/"/>\
                                </CiscoIPPhoneExecute>' %  (SERVERURL, a.pk, cli, exten)}
                        params = urllib.urlencode(xml)
                        headers = {"TE": "deflate,gzip;q=0.3",
                               "Connection": "TE, close",
                               "Authorization": "Basic YWRtaW46Y2lzY28=",
                               "User-Agent": "thunder-php-ua",
                               }
                        if a.ipaddr:
                            print >> sys.stderr, "mando il post a %s (%s)" % (a.ipaddr, a.name)
                            print >> sys.stderr, "alla url: %s/switch/cisco_updatestatus_calling/%s/%s/%s/" %  (SERVERURL, a.pk, cli, exten)
                            try:
                                conn = httplib.HTTPConnection("%s" % a.ipaddr, timeout=1)
                                conn.request("POST", "/CGI/Execute", params, headers)
                            except Exception, e:
                                print >> sys.stderr, "fallito post a %s per %s" % (a.ipaddr, str(e))
                        else:
                            print >> sys.stderr, "il device di %s non sembra essere registrato" % a.name
        except Exception, e:
            print >> sys.stderr, "Exception in DeviceCisco.pre_dial_action: %s" % str(e)
        # se sono io l'exten chiamata mando ai miei monitoranti la push
        # allo stati di ringing
        try:
            if DeviceCisco.objects.get(account__name=exten) == self:
                print >> sys.stderr, "%s si dichiara ringing a %s" % (self.account.utentepbx, self.account.utentepbx.osservatori.all())
                for u in self.account.utentepbx.osservatori.all():
                    for a in u.accounts_utentepbx.all():
                        xml = { "XML" : '<CiscoIPPhoneExecute>\
                                <ExecuteItem URL="%s/switch/cisco_updatestatus_ringing/%s/%s/%s/"/>\
                                </CiscoIPPhoneExecute>' %  (SERVERURL, a.pk, cli, exten)}
                        params = urllib.urlencode(xml)
                        headers = {"TE": "deflate,gzip;q=0.3",
                               "Connection": "TE, close",
                               "Authorization": "Basic YWRtaW46Y2lzY28=",
                               "User-Agent": "thunder-php-ua",
                               }
                        if a.ipaddr:
                            print >> sys.stderr, "mando il post a %s (%s)" % (a.ipaddr, a.name)
                            print >> sys.stderr, "alla url: %s/switch/cisco_updatestatus_ringing/%s/%s/%s/" %  (SERVERURL, a.pk, cli, exten)
                            try:
                                conn = httplib.HTTPConnection("%s" % a.ipaddr, timeout=1)
                                conn.request("POST", "/CGI/Execute", params, headers)
                            except Exception, e:
                                print >> sys.stderr, "fallito post a %s per %s" % (a.ipaddr, str(e))
                        else:
                            print >> sys.stderr, "il device di %s non sembra essere registrato" % a.name
        except Exception, e:
            print >> sys.stderr, "Exception in DeviceCisco.pre_dial_action: %s" % str(e)

    def answer_action(self, cli, exten):
        print >> sys.stderr, "INTO CISCO ANSWER %s %s" % (cli, exten)        
        # se sono io l'exten chiamata mando ai miei monitoranti la push
        # allo stato di calling
        try:
            if DeviceCisco.objects.get(account__name=exten) == self:
                print >> sys.stderr, "%s si dichiara calling a %s" % (self.account.utentepbx, self.account.utentepbx.osservatori.all())
                for u in self.account.utentepbx.osservatori.all():
                    for a in u.accounts_utentepbx.all():
                        xml = { "XML" : '<CiscoIPPhoneExecute>\
                                <ExecuteItem URL="%s/switch/cisco_updatestatus_calling/%s/%s/%s/"/>\
                                </CiscoIPPhoneExecute>' %  (SERVERURL, a.pk, cli, exten)}
                        params = urllib.urlencode(xml)
                        headers = {"TE": "deflate,gzip;q=0.3",
                                   "Connection": "TE, close",
                                   "Authorization": "Basic YWRtaW46Y2lzY28=",
                                   "User-Agent": "thunder-php-ua",
                               }
                        if a.ipaddr:
                            print >> sys.stderr, "mando il post a %s (%s)" % (a.ipaddr, a.name)
                            print >> sys.stderr, "alla url: %s/switch/cisco_updatestatus_calling/%s/%s/%s/" %  (SERVERURL, a.pk, cli, exten)
                            try:
                                conn = httplib.HTTPConnection("%s" % a.ipaddr, timeout=1)
                                conn.request("POST", "/CGI/Execute", params, headers)
                            except Exception, e:
                                print >> sys.stderr, "fallito post a %s per %s" % (a.ipaddr, str(e))
                        else:
                            print >> sys.stderr, "il device di %s non sembra essere registrato" % a.name
        except Exception, e:
            print >> sys.stderr, "Exception in DeviceCisco.answer_action: %s" % str(e)
        
    def post_dial_action(self, cli, exten):
        print >> sys.stderr, "INTO CISCO POSTDIAL %s %s" % (cli, exten)
        # mando ai miei monitoranti la push allo stato di free
        try:
            print >> sys.stderr, "%s si dichiara free a %s" % (self.account.utentepbx, self.account.utentepbx.osservatori.all())
            for u in self.account.utentepbx.osservatori.all():
                for a in u.accounts_utentepbx.all():
                    xml = { "XML" : '<CiscoIPPhoneExecute>\
                            <ExecuteItem URL="%s/switch/cisco_updatestatus_hanged/%s/%s/%s/"/>\
                            </CiscoIPPhoneExecute>' %  (SERVERURL, a.pk, cli, exten)}
                    params = urllib.urlencode(xml)
                    headers = {"TE": "deflate,gzip;q=0.3",
                               "Connection": "TE, close",
                               "Authorization": "Basic YWRtaW46Y2lzY28=",
                               "User-Agent": "thunder-php-ua",
                           }
                    if a.ipaddr:
                        print >> sys.stderr, "mando il post a %s (%s)" % (a.ipaddr, a.name)
                        print >> sys.stderr, "alla url: %s/switch/cisco_updatestatus_hanged/%s/%s/%s/" %  (SERVERURL, a.pk, cli, exten)
                        try:
                            conn = httplib.HTTPConnection("%s" % a.ipaddr, timeout=1)
                            conn.request("POST", "/CGI/Execute", params, headers)
                        except Exception, e:
                            print >> sys.stderr, "fallito post a %s per %s" % (a.ipaddr, str(e))
                    else:
                        print >> sys.stderr, "il device di %s non sembra essere registrato" % a.name
        except Exception, e:
            print >> sys.stderr, "Exception in DeviceCisco.post_dial_action: %s" % str(e)

    def restart_action(self):
        print >> sys.stderr, "INTO RESTART ACTION"
        a = self.account
        actions = (('Key:Settings', '.2'),
                   ('Key:KeyPadStar', '.05'),
                   ('Key:KeyPadStar', '.05'),
                   ('Key:KeyPadPound', '.05'),
                   ('Key:KeyPadStar', '.05'),
                   ('Key:KeyPadStar', '0'))
#        auth = base64.b64encode("admin:cisco")
        print >> sys.stderr, "mando il post di reset a %s (%s)" % (a.ipaddr, a.name)
        for action in actions:
            xml = { "XML" : '<CiscoIPPhoneExecute><ExecuteItem URL="%s"/></CiscoIPPhoneExecute>' % action[0]}
            params = urllib.urlencode(xml)
            headers = {"TE": "deflate,gzip;q=0.3",
                       "Connection": "TE, close",
                       "Authorization": "Basic YWRtaW46Y2lzY28=", #auth
                       "Host": "%s" % a.ipaddr,
                       "User-Agent": "thunder-php-ua",
                       }
    
            if a.ipaddr:
                try:
                    conn = httplib.HTTPConnection("%s" % a.ipaddr, timeout=1)
                    conn.request("POST", "/CGI/Execute", params, headers)
                except Exception, e:
                    print >> sys.stderr, "fallito post a %s per %s" % (a.ipaddr, str(e))

class DeviceCiscoDual(Device):
    account_due = models.ForeignKey(Account, blank=True, null=True)
    
    def save(self, *args, **kwargs):
        super(DeviceCiscoDual, self).save(*args, **kwargs)
        self.create_provision_file()
    
    def create_provision_file(self):
        account = self.account
        utente = account.utentepbx
        user = utente.get_user()
        account_due = self.account_due
        utente_due = account_due.utentepbx
        user_due = utente_due.get_user()
        try:
            phonelabel = "%s %s." % (user.last_name, user.first_name[0:1])
        except:
            phonelabel = 'no name set'
        if len(phonelabel) > 12:
            phonelabel = phonelabel[0:12]
        ctx_dict = {'server' : SERVERURL,
                    'processnodename' : SERVERIP,
                    'proxy' : SERVERIP,
                    'name' : self.account.name,
                    'password' : self.account.secret,
                    'phonelabel': phonelabel,
                    'name_due' : self.account_due.name,
                    'password_due' : self.account_due.secret}
        content = render_to_string('switch/cisco/provision_dual.xml', ctx_dict)
        file = open("/srv/tftp/SEP%s.cnf.xml" % self.mac_address.replace(':', '').upper(),
                    "w")
        file.write(content)
        file.close()
    
    def pre_dial_action(self, cli, exten):
        print >> sys.stderr, "INTO CISCO PREDIAL %s %s" % (cli, exten)
        # se il cli sono io mando ai miei monitoranti la push 
        # allo stato di calling
        try:
            if DeviceCiscoDual.objects.get(account__name=cli) == self:
                print >> sys.stderr, "%s si dichiara calling a %s" % (self.account.utentepbx, self.account.utentepbx.osservatori.all())
                for u in self.account.utentepbx.osservatori.all():
                    for a in u.accounts_utentepbx.all():
                        xml = { "XML" : '<CiscoIPPhoneExecute>\
                                <ExecuteItem URL="%s/switch/cisco_updatestatus_calling/%s/%s/%s/"/>\
                                </CiscoIPPhoneExecute>' %  (SERVERURL, a.pk, cli, exten)}
                        params = urllib.urlencode(xml)
                        headers = {"TE": "deflate,gzip;q=0.3",
                               "Connection": "TE, close",
                               "Authorization": "Basic YWRtaW46Y2lzY28=",
                               "User-Agent": "thunder-php-ua",
                               }
                        if a.ipaddr:
                            print >> sys.stderr, "mando il post a %s (%s)" % (a.ipaddr, a.name)
                            print >> sys.stderr, "alla url: %s/switch/cisco_updatestatus_calling/%s/%s/%s/" %  (SERVERURL, a.pk, cli, exten)
                            try:
                                conn = httplib.HTTPConnection("%s" % a.ipaddr, timeout=1)
                                conn.request("POST", "/CGI/Execute", params, headers)
                            except Exception, e:
                                print >> sys.stderr, "fallito post a %s per %s" % (a.ipaddr, str(e))
                        else:
                            print >> sys.stderr, "il device di %s non sembra essere registrato" % a.name
        except Exception, e:
            print >> sys.stderr, "Exception in DeviceCiscoDual.pre_dial_action: %s" % str(e)
        # se sono io l'exten chiamata mando ai miei monitoranti la push
        # allo stati di ringing
        try:
            if DeviceCiscoDual.objects.get(account__name=exten) == self:
                print >> sys.stderr, "%s si dichiara ringing a %s" % (self.account.utentepbx, self.account.utentepbx.osservatori.all())
                for u in self.account.utentepbx.osservatori.all():
                    for a in u.accounts_utentepbx.all():
                        xml = { "XML" : '<CiscoIPPhoneExecute>\
                                <ExecuteItem URL="%s/switch/cisco_updatestatus_ringing/%s/%s/%s/"/>\
                                </CiscoIPPhoneExecute>' %  (SERVERURL, a.pk, cli, exten)}
                        params = urllib.urlencode(xml)
                        headers = {"TE": "deflate,gzip;q=0.3",
                               "Connection": "TE, close",
                               "Authorization": "Basic YWRtaW46Y2lzY28=",
                               "User-Agent": "thunder-php-ua",
                               }
                        if a.ipaddr:
                            print >> sys.stderr, "mando il post a %s (%s)" % (a.ipaddr, a.name)
                            print >> sys.stderr, "alla url: %s/switch/cisco_updatestatus_ringing/%s/%s/%s/" %  (SERVERURL, a.pk, cli, exten)
                            try:
                                conn = httplib.HTTPConnection("%s" % a.ipaddr, timeout=1)
                                conn.request("POST", "/CGI/Execute", params, headers)
                            except Exception, e:
                                print >> sys.stderr, "fallito post a %s per %s" % (a.ipaddr, str(e))
                        else:
                            print >> sys.stderr, "il device di %s non sembra essere registrato" % a.name
        except Exception, e:
            print >> sys.stderr, "Exception in DeviceCiscoDual.pre_dial_action: %s" % str(e)

    def answer_action(self, cli, exten):
        print >> sys.stderr, "INTO CISCO ANSWER %s %s" % (cli, exten)        
        # se sono io l'exten chiamata mando ai miei monitoranti la push
        # allo stato di calling
        try:
            if DeviceCiscoDual.objects.get(account__name=exten) == self:
                print >> sys.stderr, "%s si dichiara calling a %s" % (self.account.utentepbx, self.account.utentepbx.osservatori.all())
                for u in self.account.utentepbx.osservatori.all():
                    for a in u.accounts_utentepbx.all():
                        xml = { "XML" : '<CiscoIPPhoneExecute>\
                                <ExecuteItem URL="%s/switch/cisco_updatestatus_calling/%s/%s/%s/"/>\
                                </CiscoIPPhoneExecute>' %  (SERVERURL, a.pk, cli, exten)}
                        params = urllib.urlencode(xml)
                        headers = {"TE": "deflate,gzip;q=0.3",
                                   "Connection": "TE, close",
                                   "Authorization": "Basic YWRtaW46Y2lzY28=",
                                   "User-Agent": "thunder-php-ua",
                               }
                        if a.ipaddr:
                            print >> sys.stderr, "mando il post a %s (%s)" % (a.ipaddr, a.name)
                            print >> sys.stderr, "alla url: %s/switch/cisco_updatestatus_calling/%s/%s/%s/" %  (SERVERURL, a.pk, cli, exten)
                            try:
                                conn = httplib.HTTPConnection("%s" % a.ipaddr, timeout=1)
                                conn.request("POST", "/CGI/Execute", params, headers)
                            except Exception, e:
                                print >> sys.stderr, "fallito post a %s per %s" % (a.ipaddr, str(e))
                        else:
                            print >> sys.stderr, "il device di %s non sembra essere registrato" % a.name
        except Exception, e:
            print >> sys.stderr, "Exception in DeviceCiscoDual.answer_action: %s" % str(e)
        
    def post_dial_action(self, cli, exten):
        print >> sys.stderr, "INTO CISCO POSTDIAL %s %s" % (cli, exten)
        # mando ai miei monitoranti la push allo stato di free
        try:
            print >> sys.stderr, "%s si dichiara free a %s" % (self.account.utentepbx, self.account.utentepbx.osservatori.all())
            for u in self.account.utentepbx.osservatori.all():
                for a in u.accounts_utentepbx.all():
                    xml = { "XML" : '<CiscoIPPhoneExecute>\
                            <ExecuteItem URL="%s/switch/cisco_updatestatus_hanged/%s/%s/%s/"/>\
                            </CiscoIPPhoneExecute>' %  (SERVERURL, a.pk, cli, exten)}
                    params = urllib.urlencode(xml)
                    headers = {"TE": "deflate,gzip;q=0.3",
                               "Connection": "TE, close",
                               "Authorization": "Basic YWRtaW46Y2lzY28=",
                               "User-Agent": "thunder-php-ua",
                           }
                    if a.ipaddr:
                        print >> sys.stderr, "mando il post a %s (%s)" % (a.ipaddr, a.name)
                        print >> sys.stderr, "alla url: %s/switch/cisco_updatestatus_hanged/%s/%s/%s/" %  (SERVERURL, a.pk, cli, exten)
                        try:
                            conn = httplib.HTTPConnection("%s" % a.ipaddr, timeout=1)
                            conn.request("POST", "/CGI/Execute", params, headers)
                        except Exception, e:
                            print >> sys.stderr, "fallito post a %s per %s" % (a.ipaddr, str(e))
                    else:
                        print >> sys.stderr, "il device di %s non sembra essere registrato" % a.name
        except Exception, e:
            print >> sys.stderr, "Exception in DeviceCiscoDual.post_dial_action: %s" % str(e)

    def restart_action(self):
        print >> sys.stderr, "INTO RESTART ACTION"
        a = self.account
        actions = (('Key:Settings', '.2'),
                   ('Key:KeyPadStar', '.05'),
                   ('Key:KeyPadStar', '.05'),
                   ('Key:KeyPadPound', '.05'),
                   ('Key:KeyPadStar', '.05'),
                   ('Key:KeyPadStar', '0'))
#        auth = base64.b64encode("admin:cisco")
    
        for action in actions:
            xml = { "XML" : '<CiscoIPPhoneExecute><ExecuteItem URL="%s"/></CiscoIPPhoneExecute>' % action}
            params = urllib.urlencode(xml)
            headers = {"TE": "deflate,gzip;q=0.3",
                                   "Connection": "TE, close",
                                   "Authorization": "Basic YWRtaW46Y2lzY28=", #auth
                                   "Host": "%s" % a.ipaddr,
                                   "User-Agent": "thunder-php-ua",
                               }
    
            if a.ipaddr:
                print >> sys.stderr, "mando il post di reset a %s (%s)" % (a.ipaddr, a.name)
                try:
                    conn = httplib.HTTPConnection("%s" % a.ipaddr, timeout=1)
                    conn.request("POST", "/CGI/Execute", params, headers)
                except Exception, e:
                    print >> sys.stderr, "fallito post a %s per %s" % (a.ipaddr, str(e))