# -*- coding: utf-8 -*-
'''
Created on 28/gen/2012

@author: simone cittadini
'''
from django.contrib import admin
from django import forms
from django.utils.translation import ugettext_lazy as _
from django.conf.urls.defaults import patterns, url
from django.core.urlresolvers import reverse

from jumbo.admin.options import ExtendibleModelAdmin

from pbx.switch.views import device_restart
from pbx.switch.models import Account
from models import DeviceDummy


class DeviceDummySearchForm(forms.ModelForm):
    class Meta:
        model = Account
        display_fields = (
            'mac_address__icontains','account__name__icontains',
        )

    def __init__(self, *args, **kws):
        super(DeviceDummySearchForm, self).__init__(*args, **kws)
        self.fields['mac_address__icontains'] = forms.CharField(label=_("Mac Address"), required=False)
        self.fields['account__name__icontains'] = forms.CharField(label=_("Numero account"), required=False)

class DeviceDummyAdmin(ExtendibleModelAdmin):
    list_display = ('nome','mac_address', 'get_restart_icon',)

    search_form = DeviceDummySearchForm
    no_display_links = False
    
    def get_urls(self):
        urls = super(DeviceDummyAdmin, self).get_urls()
        my_urls = patterns('',
                           url(r'^device_restart/(?P<device_id>\d+)$',
                               self.admin_site.admin_view(device_restart),
                               name='device_restart'),
                           )
        return my_urls + urls
    
    def get_restart_icon(self, obj):
        rev = reverse('admin:device_restart', args=(obj.id,))
        return "<a href='%s' title='restart hw'><img src='/static/admin/images/url_icon.png' /></a>" % (rev,)

    get_restart_icon.allow_tags = True
    get_restart_icon.short_description = 'restart'

admin.site.register(DeviceDummy, DeviceDummyAdmin)
