# -*- coding: utf-8 -*-
'''
Created on 1/ago/2011

@author: simone cittadini
'''
from django.db import models
from django.db.models.signals import post_save
#from jumbo.apps.auth.models import JumboUser
from django.contrib.auth.models import User

from jumbo.apps.profile.models import SCProfiles, SCProfile


CHOICES_VERSO_BL = (('IN', 'IN'),
                    ('OUT', 'OUT'),
                    ('BOTH', 'BOTH'),)

class Reparto(models.Model):
    nome = models.CharField(max_length=128, unique=True)
    
    def __unicode__(self):
        return self.nome
    
    class Meta:
        verbose_name_plural='Reparti'
        
class PbxProfiles(SCProfiles):
    pass

    def __unicode__(self):
        return str(self.user)

class UtentePBX(SCProfile):
    nome = models.CharField(max_length=64)
    is_admin = models.BooleanField(default=False)
    utenti_monitorati = models.ManyToManyField('self',
                                               symmetrical=False,
                                               related_name='osservatori',
                                               blank=True, null=True)
    reparti = models.ManyToManyField(Reparto, related_name='utentipbx',
                                     blank=True, null=True)
    warning_chiamate_perse = models.BooleanField(default=False)
    
    @property
    def cognome(self):
        return self.get_user().last_name
    
    def __unicode__(self):
        return self.nome
    
    class Meta:
        verbose_name_plural='Utenti PBX'
        ordering = ('nome',)
        
class PersonalPhonebookEntry(models.Model):
    utentepbx = models.ForeignKey(UtentePBX)
    numero = models.CharField(max_length=64)
    nome = models.CharField(max_length=64)
    
    class Meta:
        ordering = ['nome']
    
    def __unicode__(self):
        return "%s - %s" % (self.numero, self.nome)
    
class BlacklistEntry(models.Model):
    numero = models.CharField(max_length=64)
    nome = models.CharField(max_length=64)
    verso = models.CharField(max_length=4, choices=CHOICES_VERSO_BL,
                             default='BOTH')
    
    class Meta:
        ordering = ('verso', 'nome', 'numero')
    
    def __unicode__(self):
        return "%s - %s (%s)" % (self.verso, self.numero, self.nome)
    
class SpeedDialEntry(models.Model):
    numero = models.CharField(max_length=3)
    destinazione = models.CharField(max_length=64)
    nome = models.CharField(max_length=64)
    
    class Meta:
        ordering = ('numero', 'nome')
        
    def __unicode__(self):
        return "%s - %s (%s)" % (self.numero, self.nome, self.destinazione)
        
class EnabledCookie(models.Model):
    value = models.CharField(max_length=64, unique=True)
    timestamp = models.DateTimeField(auto_now=True)
    enabled = models.BooleanField(default=False)
        
def create_user_profile(sender, instance, created, **kwargs):
    if created:
        profiles = PbxProfiles.objects.create(user=instance)
        profiles.add_profile("UtentePBX", "pbx.utenti.models",
                             {'nome' : instance.username})
        instance.is_staff = True
        instance.save()
        
post_save.connect(create_user_profile, sender=User)
